
module TEST_COLOR_CHAR_MODE (
//CLOCK,
VGA_OUT_PIXEL_CLOCK,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH,
VGA_VSYNCH,
VGA_OUT_RED,
VGA_OUT_GREEN,
VGA_OUT_BLUE,
);
   `define top_clk_per 100
   `define Max_Sim_Time 1000000000     // stop simulation after this time (ns)
	reg CLOCK;
   always #(`top_clk_per/2) CLOCK = ~CLOCK;

   initial begin
      // Initialize TB signals
      CLOCK = 1'b0;
      //top_reset = 1'b1;
      
      //#800;  
      //$display ("\nReset asserted at time = %t", $realtime);
      //top_reset = 1'b0;
      //#800;
      //top_reset = 1'b1;     
   end
//input 			CLOCK;					// 100MHz system clock	
output 			VGA_OUT_PIXEL_CLOCK;	// pixel clock for the video DAC
output			VGA_COMP_SYNCH_N;		// composite sync for the video DAC
output 			VGA_OUT_BLANK_N;		// composite blanking for the video DAC
output 			VGA_HSYNCH;				// horizontal sync for the VGA output connector
output			VGA_VSYNCH;				// vertical sync for the VGA output connector
output [7:0] 	VGA_OUT_RED;			// RED DAC data
output [7:0] 	VGA_OUT_GREEN;			// GREEN DAC data
output [7:0] 	VGA_OUT_BLUE;			// BLUE DAC data


wire 			VGA_OUT_PIXEL_CLOCK;
wire 			VGA_COMP_SYNCH_N;
wire 			VGA_OUT_BLANK_N;
wire 			VGA_HSYNCH;
wire 			VGA_VSYNCH;
wire [7:0]		VGA_OUT_RED;
wire [7:0]		VGA_OUT_GREEN;
wire [7:0]		VGA_OUT_BLUE;

//wire CLOCK;
wire clock;
wire clk_100;
wire clock_100;
wire clk_25;
wire clock_25;
wire clk_40;
wire clock_40;
wire system_dcm_rst;
wire system_dcm_locked;
wire reset = !system_dcm_locked;
wire pixel_clock = clk_40;

wire low = 1'b0;
wire high =1'b1;
reg [12:0] char_write_addr;
wire [12:0] char_write_addr_dummy;
reg [11:0] color_char_write_data;
reg [6:0] character_counter;
reg [6:0] line_counter;
wire char_write_clock = clk_25;
wire char_write_enable = high;
wire [3:0] color_data = color_char_write_data[11:8];
wire [7:0] char_data =  color_char_write_data[7:0]; 
// instantiate the clock input buffers for the single ended clocks
//IBUFG CLOCK_INPUT_BUF (
//.O  (clock),
//.I  (CLOCK)
//);
assign clock = CLOCK;
// instantiate the clock input buffers for the internal clocks
BUFG CLK_100MHZ_BUF (
.O  (clk_100),
.I  (clock_100)
);

BUFG CLK_25MHZ_BUF (
.O  (clk_25),
.I  (clock_25)
);

BUFG CLK_40MHZ_BUF (
.O  (clk_40),
.I  (clock_40)
);

// instantiate the DCMs and DCM reset generation
DCM SYSTEM_DCM (
.CLKFB		(clk_100),
.CLKIN		(clock),
.DSSEN		(low),
.PSCLK		(low),
.PSEN		(low),
.PSINCDEC	(low),
.RST		(system_dcm_rst),
.CLK0		(clock_100),
.CLK90		(),
.CLK180		(),
.CLK270		(),
.CLK2X		(),
.CLK2X180	(),
.CLKDV		(clock_25),
.CLKFX		(clock_40),
.CLKFX180	(),
.LOCKED		(system_dcm_locked),
.PSDONE		(),
.STATUS		()
)
/* synthesis xc_props="DLL_FREQUENCY_MODE =LOW,DUTY_CYCLE_CORRECTION =TRUE,STARTUP_WAIT =FALSE,DFS_FREQUENCY_MODE =LOW,CLKFX_DIVIDE =10,CLKFX_MULTIPLY =4,CLKDV_DIVIDE =4,CLK_FEEDBACK =1X,CLKOUT_PHASE_SHIFT =NONE,PHASE_SHIFT =0"*/;


SRL16 RESET_SYSTEM_DCM (
.Q (system_dcm_rst),
.CLK (clock),
.D (low),
.A0 (high),
.A1 (high),
.A2 (high),
.A3 (high)
)/*synthesis xc_props="INIT = 000F"*/;


// instantiate the SVGA controller and display RAM
color_char_mode_svga_ctrl COLOR_CHAR_MODE_SVGA_CTRL(
VGA_OUT_PIXEL_CLOCK,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH,
VGA_VSYNCH,
VGA_OUT_RED,
VGA_OUT_GREEN,
VGA_OUT_BLUE,

char_write_addr_dummy,
char_data,
char_write_enable,
char_write_clock,

//color_data,

pixel_clock,
reset
);



// create a character counter
always @ ( posedge char_write_clock or posedge reset) begin
	if (reset ) begin
		character_counter <= 7'h00;
		end
	else if (character_counter == 99) begin
		character_counter <= 7'h00;
		end	
	else begin
		character_counter <= character_counter + 1;
		end
	end

// create a line counter
always @ ( posedge char_write_clock or posedge reset) begin
	if (reset ) begin
		line_counter <= 7'h00;
		end
	else if ((line_counter == 74)&(character_counter == 99)) begin
		line_counter <= 7'h00;
		end
	else if(character_counter == 99) begin
		line_counter <= line_counter + 1;
		end	
	end

// create the write address counter
always @ ( posedge char_write_clock or posedge reset) begin
	if (reset) begin
		char_write_addr <= 13'h0000;
		end
	else if ((line_counter == 74)&(character_counter == 99)) begin
		char_write_addr <= 13'h0000;
		end
	else begin
		char_write_addr <= char_write_addr +1;
		end
	end

// create the display data
always @ ( posedge char_write_clock or posedge reset) begin
	if (reset) begin
		color_char_write_data <= 12'h020;	
		end
	else if (line_counter == 2) begin
		if (character_counter == 22) begin
			color_char_write_data <= 12'hF80;	//X	
			end
		else if (character_counter == 23) begin
			color_char_write_data <= 12'hF81;	
			end
		else if (character_counter == 24) begin	//U
			color_char_write_data <= 12'hF84;	
			end
		else if (character_counter == 25) begin
			color_char_write_data <= 12'hF85;	
			end
		else if (character_counter == 26) begin	//P
			color_char_write_data <= 12'hF88;	
			end
		else if (character_counter == 27) begin
			color_char_write_data <= 12'hF89;	
			end
		else if (character_counter == 28) begin	//-
			color_char_write_data <= 12'hF8C;	
			end
		else if (character_counter == 29) begin
			color_char_write_data <= 12'hF8D;	
			end
		else if (character_counter == 30) begin	//V
			color_char_write_data <= 12'hF90;	
			end
		else if (character_counter == 31) begin
			color_char_write_data <= 12'hF91;	
			end
		else if (character_counter == 32) begin	//2
			color_char_write_data <= 12'hF94;	
			end
		else if (character_counter == 33) begin
			color_char_write_data <= 12'hF95;	
			end
		else if (character_counter == 34) begin	//P
			color_char_write_data <= 12'hF88;	
			end
		else if (character_counter == 35) begin
			color_char_write_data <= 12'hF89;	
			end
		else if (character_counter == 36) begin	//r
			color_char_write_data <= 12'hF98;	
			end
		else if (character_counter == 37) begin
			color_char_write_data <= 12'hF99;	
			end
		else if (character_counter == 38) begin	//o
			color_char_write_data <= 12'hF9C;	
			end
		else if (character_counter == 39) begin
			color_char_write_data <= 12'hF9D;	
			end
		else if (character_counter == 40) begin	//space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 41) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 42) begin	//D
			color_char_write_data <= 12'hFA0;	
			end
		else if (character_counter == 43) begin
			color_char_write_data <= 12'hFA1;	
			end
		else if (character_counter == 44) begin	//e
			color_char_write_data <= 12'hFA4;	
			end
		else if (character_counter == 45) begin
			color_char_write_data <= 12'hFA5;	
			end
		else if (character_counter == 46) begin //v
			color_char_write_data <= 12'hFA8;	
			end
		else if (character_counter == 47) begin
			color_char_write_data <= 12'hFA9;	
			end
		else if (character_counter == 48) begin //e
			color_char_write_data <= 12'hFA4;	
			end
		else if (character_counter == 49) begin
			color_char_write_data <= 12'hFA5;	
			end
		else if (character_counter == 50) begin //l
			color_char_write_data <= 12'hFAC;	
			end
		else if (character_counter == 51) begin
			color_char_write_data <= 12'hFAD;	
			end
		else if (character_counter == 52) begin	//o
			color_char_write_data <= 12'hF9C;	
			end
		else if (character_counter == 53) begin
			color_char_write_data <= 12'hF9D;	
			end
		else if (character_counter == 54) begin	//p
			color_char_write_data <= 12'hFB0;	
			end
		else if (character_counter == 55) begin
			color_char_write_data <= 12'hFB1;	
			end
		else if (character_counter == 56) begin	//m
			color_char_write_data <= 12'hFB4;	
			end
		else if (character_counter == 57) begin	
			color_char_write_data <= 12'hFB5;	
			end
		else if (character_counter == 58) begin	//e
			color_char_write_data <= 12'hFA4;	
			end
		else if (character_counter == 59) begin
			color_char_write_data <= 12'hFA5;	
			end
		else if (character_counter == 60) begin	//n
			color_char_write_data <= 12'hFB8;	
			end
		else if (character_counter == 61) begin
			color_char_write_data <= 12'hFB9;	
			end
		else if (character_counter == 62) begin	//t
			color_char_write_data <= 12'hFBC;	
			end
		else if (character_counter == 63) begin	
			color_char_write_data <= 12'hFBD;	
			end
		else if (character_counter == 64) begin	// space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 65) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 66) begin	//S
			color_char_write_data <= 12'hFC0;	
			end
		else if (character_counter == 67) begin
			color_char_write_data <= 12'hFC1;	
			end
		else if (character_counter == 68) begin	//y
			color_char_write_data <= 12'hFC4;	
			end
		else if (character_counter == 69) begin
			color_char_write_data <= 12'hFC5;	
			end
		else if (character_counter == 70) begin	//s
			color_char_write_data <= 12'hFC8;	
			end
		else if (character_counter == 71) begin
			color_char_write_data <= 12'hFC9;	
			end
		else if (character_counter == 72) begin	//t
			color_char_write_data <= 12'hFBC;	
			end
		else if (character_counter == 73) begin
			color_char_write_data <= 12'hFBD;	
			end
		else if (character_counter == 74) begin	//e
			color_char_write_data <= 12'hFA4;	
			end
		else if (character_counter == 75) begin
			color_char_write_data <= 12'hFA5;	
			end
		else if (character_counter == 76) begin //m
			color_char_write_data <= 12'hFB4;	
			end
		else if (character_counter == 77) begin
			color_char_write_data <= 12'hFB5;	
			end
		else begin
			color_char_write_data <= 12'h020;	
			end
		end

	else if (line_counter == 3) begin
		if (character_counter == 22) begin
			color_char_write_data <= 12'h782;	//X	
			end
		else if (character_counter == 23) begin
			color_char_write_data <= 12'h783;	
			end
		else if (character_counter == 24) begin	//U
			color_char_write_data <= 12'h786;	
			end
		else if (character_counter == 25) begin
			color_char_write_data <= 12'h787;	
			end
		else if (character_counter == 26) begin	//P
			color_char_write_data <= 12'h78A;	
			end
		else if (character_counter == 27) begin
			color_char_write_data <= 12'h78B;	
			end
		else if (character_counter == 28) begin	//-
			color_char_write_data <= 12'h78E;	
			end
		else if (character_counter == 29) begin
			color_char_write_data <= 12'h78F;	
			end
		else if (character_counter == 30) begin	//V
			color_char_write_data <= 12'h792;	
			end
		else if (character_counter == 31) begin
			color_char_write_data <= 12'h793;	
			end
		else if (character_counter == 32) begin	//2
			color_char_write_data <= 12'h796;	
			end
		else if (character_counter == 33) begin
			color_char_write_data <= 12'h797;	
			end
		else if (character_counter == 34) begin	//P
			color_char_write_data <= 12'h78A;	
			end
		else if (character_counter == 35) begin
			color_char_write_data <= 12'h78B;	
			end
		else if (character_counter == 36) begin	//r
			color_char_write_data <= 12'h79A;	
			end
		else if (character_counter == 37) begin
			color_char_write_data <= 12'h79B;	
			end
		else if (character_counter == 38) begin	//o
			color_char_write_data <= 12'h79E;	
			end
		else if (character_counter == 39) begin
			color_char_write_data <= 12'h79F;	
			end
		else if (character_counter == 40) begin	//space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 41) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 42) begin	//D
			color_char_write_data <= 12'h7A2;	
			end
		else if (character_counter == 43) begin
			color_char_write_data <= 12'h7A3;	
			end
		else if (character_counter == 44) begin	//e
			color_char_write_data <= 12'h7A6;	
			end
		else if (character_counter == 45) begin
			color_char_write_data <= 12'h7A7;	
			end
		else if (character_counter == 46) begin //v
			color_char_write_data <= 12'h7AA;	
			end
		else if (character_counter == 47) begin
			color_char_write_data <= 12'h7AB;	
			end
		else if (character_counter == 48) begin //e
			color_char_write_data <= 12'h7A6;	
			end
		else if (character_counter == 49) begin
			color_char_write_data <= 12'h7A7;	
			end
		else if (character_counter == 50) begin //l
			color_char_write_data <= 12'h7AE;	
			end
		else if (character_counter == 51) begin
			color_char_write_data <= 12'h7AF;	
			end
		else if (character_counter == 52) begin	//o
			color_char_write_data <= 12'h79E;	
			end
		else if (character_counter == 53) begin
			color_char_write_data <= 12'h79F;	
			end
		else if (character_counter == 54) begin	//p
			color_char_write_data <= 12'h7B2;	
			end
		else if (character_counter == 55) begin
			color_char_write_data <= 12'h7B3;	
			end
		else if (character_counter == 56) begin	//m
			color_char_write_data <= 12'h7B6;	
			end
		else if (character_counter == 57) begin	
			color_char_write_data <= 12'h7B7;	
			end
		else if (character_counter == 58) begin	//e
			color_char_write_data <= 12'h7A6;	
			end
		else if (character_counter == 59) begin
			color_char_write_data <= 12'h7A7;	
			end
		else if (character_counter == 60) begin	//n
			color_char_write_data <= 12'h7BA;	
			end
		else if (character_counter == 61) begin
			color_char_write_data <= 12'h7BB;	
			end
		else if (character_counter == 62) begin	//t
			color_char_write_data <= 12'h7BE;	
			end
		else if (character_counter == 63) begin	
			color_char_write_data <= 12'h7BF;	
			end
		else if (character_counter == 64) begin	// space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 65) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 66) begin	//S
			color_char_write_data <= 12'h7C2;	
			end
		else if (character_counter == 67) begin
			color_char_write_data <= 12'h7C3;	
			end
		else if (character_counter == 68) begin	//y
			color_char_write_data <= 12'h7C6;	
			end
		else if (character_counter == 69) begin
			color_char_write_data <= 12'h7C7;	
			end
		else if (character_counter == 70) begin	//s
			color_char_write_data <= 12'h7CA;	
			end
		else if (character_counter == 71) begin
			color_char_write_data <= 12'h7CB;	
			end
		else if (character_counter == 72) begin	//t
			color_char_write_data <= 12'h7BE;	
			end
		else if (character_counter == 73) begin
			color_char_write_data <= 12'h7BF;	
			end
		else if (character_counter == 74) begin	//e
			color_char_write_data <= 12'h7A6;	
			end
		else if (character_counter == 75) begin
			color_char_write_data <= 12'h7A7;	
			end
		else if (character_counter == 76) begin //m
			color_char_write_data <= 12'h7B6;	
			end
		else if (character_counter == 77) begin
			color_char_write_data <= 12'h7B7;	
			end
		else begin
			color_char_write_data <= 12'h020;	
			end
		end


	else if (line_counter == 5) begin
		if (character_counter == 23) begin
			color_char_write_data <= 12'hFCC;	//A	
			end
		else if (character_counter == 24) begin
			color_char_write_data <= 12'hFCD;	
			end
		else if (character_counter == 25) begin	//u
			color_char_write_data <= 12'hFD0;	
			end
		else if (character_counter == 26) begin
			color_char_write_data <= 12'hFD1;	
			end
		else if (character_counter == 27) begin	//d
			color_char_write_data <= 12'hFD4;	
			end
		else if (character_counter == 28) begin
			color_char_write_data <= 12'hFD5;	
			end
		else if (character_counter == 29) begin	//i
			color_char_write_data <= 12'hFD8;	
			end
		else if (character_counter == 30) begin
			color_char_write_data <= 12'hFD9;	
			end
		else if (character_counter == 31) begin	//o
			color_char_write_data <= 12'hF9C;	
			end
		else if (character_counter == 32) begin
			color_char_write_data <= 12'hF9D;	
			end
		else if (character_counter == 33) begin	//space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 34) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 35) begin	//F
			color_char_write_data <= 12'hFDC;	
			end
		else if (character_counter == 36) begin
			color_char_write_data <= 12'hFDD;	
			end
		else if (character_counter == 37) begin	//i
			color_char_write_data <= 12'hFD8;	
			end
		else if (character_counter == 38) begin
			color_char_write_data <= 12'hFD9;	
			end
		else if (character_counter == 39) begin	//l
			color_char_write_data <= 12'hFAC;	
			end
		else if (character_counter == 40) begin
			color_char_write_data <= 12'hFAD;	
			end
		else if (character_counter == 41) begin	//t
			color_char_write_data <= 12'hFBC;	
			end
		else if (character_counter == 42) begin
			color_char_write_data <= 12'hFBD;	
			end
		else if (character_counter == 43) begin	//e
			color_char_write_data <= 12'hFA4;	
			end
		else if (character_counter == 44) begin
			color_char_write_data <= 12'hFA5;	
			end
		else if (character_counter == 45) begin	//r
			color_char_write_data <= 12'hF98;	
			end
		else if (character_counter == 46) begin
			color_char_write_data <= 12'hF99;	
			end
		else if (character_counter == 47) begin //space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 48) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 49) begin //D
			color_char_write_data <= 12'hFA0;	
			end
		else if (character_counter == 50) begin
			color_char_write_data <= 12'hFA1;	
			end
		else if (character_counter == 51) begin //e
			color_char_write_data <= 12'hFA4;	
			end
		else if (character_counter == 52) begin
			color_char_write_data <= 12'hFA5;	
			end
		else if (character_counter == 53) begin	//m
			color_char_write_data <= 12'hFB4;	
			end
		else if (character_counter == 54) begin
			color_char_write_data <= 12'hFB5;	
			end
		else if (character_counter == 55) begin	//o
			color_char_write_data <= 12'hF9C;	
			end
		else if (character_counter == 56) begin
			color_char_write_data <= 12'hF9D;	
			end
		else if (character_counter == 57) begin	//n
			color_char_write_data <= 12'hFB8;	
			end
		else if (character_counter == 58) begin	
			color_char_write_data <= 12'hFB9;	
			end
		else if (character_counter == 59) begin	//s
			color_char_write_data <= 12'hFC8;	
			end
		else if (character_counter == 60) begin
			color_char_write_data <= 12'hFC9;	
			end
		else if (character_counter == 61) begin	//t
			color_char_write_data <= 12'hFBC;	
			end
		else if (character_counter == 62) begin
			color_char_write_data <= 12'hFBD;	
			end
		else if (character_counter == 63) begin	//r
			color_char_write_data <= 12'hF98;	
			end
		else if (character_counter == 64) begin	
			color_char_write_data <= 12'hF99;	
			end
		else if (character_counter == 65) begin	//a
			color_char_write_data <= 12'hFE0;	
			end
		else if (character_counter == 66) begin
			color_char_write_data <= 12'hFE1;	
			end
		else if (character_counter == 67) begin	//t
			color_char_write_data <= 12'hFBC;	
			end
		else if (character_counter == 68) begin
			color_char_write_data <= 12'hFBD;	
			end
		else if (character_counter == 69) begin	//i
			color_char_write_data <= 12'hFD8;	
			end
		else if (character_counter == 70) begin
			color_char_write_data <= 12'hFD9;	
			end
		else if (character_counter == 71) begin	//o
			color_char_write_data <= 12'hF9C;	
			end
		else if (character_counter == 72) begin
			color_char_write_data <= 12'hF9D;	
			end
		else if (character_counter == 73) begin	//n
			color_char_write_data <= 12'hFB8;	
			end
		else if (character_counter == 74) begin
			color_char_write_data <= 12'hFB9;	
			end
		else begin
			color_char_write_data <= 12'h020;	
			end
		end



	else if (line_counter == 6) begin
		if (character_counter == 23) begin
			color_char_write_data <= 12'h7CE;	//A	
			end
		else if (character_counter == 24) begin
			color_char_write_data <= 12'h7CF;	
			end
		else if (character_counter == 25) begin	//u
			color_char_write_data <= 12'h7D2;	
			end
		else if (character_counter == 26) begin
			color_char_write_data <= 12'h7D3;	
			end
		else if (character_counter == 27) begin	//d
			color_char_write_data <= 12'h7D6;	
			end
		else if (character_counter == 28) begin
			color_char_write_data <= 12'h7D7;	
			end
		else if (character_counter == 29) begin	//i
			color_char_write_data <= 12'h7DA;	
			end
		else if (character_counter == 30) begin
			color_char_write_data <= 12'h7DB;	
			end
		else if (character_counter == 31) begin	//o
			color_char_write_data <= 12'h79E;	
			end
		else if (character_counter == 32) begin
			color_char_write_data <= 12'h79F;	
			end
		else if (character_counter == 33) begin	//space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 34) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 35) begin	//F
			color_char_write_data <= 12'h7DE;	
			end
		else if (character_counter == 36) begin
			color_char_write_data <= 12'h7DF;	
			end
		else if (character_counter == 37) begin	//i
			color_char_write_data <= 12'h7DA;	
			end
		else if (character_counter == 38) begin
			color_char_write_data <= 12'h7DB;	
			end
		else if (character_counter == 39) begin	//l
			color_char_write_data <= 12'h7AE;	
			end
		else if (character_counter == 40) begin
			color_char_write_data <= 12'h7AF;	
			end
		else if (character_counter == 41) begin	//t
			color_char_write_data <= 12'h7BE;	
			end
		else if (character_counter == 42) begin
			color_char_write_data <= 12'h7BF;	
			end
		else if (character_counter == 43) begin	//e
			color_char_write_data <= 12'h7A6;	
			end
		else if (character_counter == 44) begin
			color_char_write_data <= 12'h7A7;	
			end
		else if (character_counter == 45) begin	//r
			color_char_write_data <= 12'h79A;	
			end
		else if (character_counter == 46) begin
			color_char_write_data <= 12'h79B;	
			end
		else if (character_counter == 47) begin //space
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 48) begin
			color_char_write_data <= 12'h020;	
			end
		else if (character_counter == 49) begin //D
			color_char_write_data <= 12'h7A2;	
			end
		else if (character_counter == 50) begin
			color_char_write_data <= 12'h7A3;	
			end
		else if (character_counter == 51) begin //e
			color_char_write_data <= 12'h7A6;	
			end
		else if (character_counter == 52) begin
			color_char_write_data <= 12'h7A7;	
			end
		else if (character_counter == 53) begin	//m
			color_char_write_data <= 12'h7B6;	
			end
		else if (character_counter == 54) begin
			color_char_write_data <= 12'h7B7;	
			end
		else if (character_counter == 55) begin	//o
			color_char_write_data <= 12'h79E;	
			end
		else if (character_counter == 56) begin
			color_char_write_data <= 12'h79F;	
			end
		else if (character_counter == 57) begin	//n
			color_char_write_data <= 12'h7BA;	
			end
		else if (character_counter == 58) begin	
			color_char_write_data <= 12'h7BB;	
			end
		else if (character_counter == 59) begin	//s
			color_char_write_data <= 12'h7CA;	
			end
		else if (character_counter == 60) begin
			color_char_write_data <= 12'h7CB;	
			end
		else if (character_counter == 61) begin	//t
			color_char_write_data <= 12'h7BE;	
			end
		else if (character_counter == 62) begin
			color_char_write_data <= 12'h7BF;	
			end
		else if (character_counter == 63) begin	//r
			color_char_write_data <= 12'h79A;	
			end
		else if (character_counter == 64) begin	
			color_char_write_data <= 12'h79B;	
			end
		else if (character_counter == 65) begin	//a
			color_char_write_data <= 12'h7E2;	
			end
		else if (character_counter == 66) begin
			color_char_write_data <= 12'h7E3;	
			end
		else if (character_counter == 67) begin	//t
			color_char_write_data <= 12'h7BE;	
			end
		else if (character_counter == 68) begin
			color_char_write_data <= 12'h7BF;	
			end
		else if (character_counter == 69) begin	//i
			color_char_write_data <= 12'h7DA;	
			end
		else if (character_counter == 70) begin
			color_char_write_data <= 12'h7DB;	
			end
		else if (character_counter == 71) begin	//o
			color_char_write_data <= 12'h79E;	
			end
		else if (character_counter == 72) begin
			color_char_write_data <= 12'h79F;	
			end
		else if (character_counter == 73) begin	//n
			color_char_write_data <= 12'h7BA;	
			end
		else if (character_counter == 74) begin
			color_char_write_data <= 12'h7BB;	
			end
		else begin
			color_char_write_data <= 12'h020;	
			end
		end


		else begin
			color_char_write_data <= 12'h020;	
			end
		end












endmodule

module SRL16(Q, A0, A1, A2, A3, CLK, D); // synthesis syn_black_box
output Q;
input A0;
input A1;
input A2;
input A3;
input CLK;
input D;
endmodule

module IBUFG(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module BUFG(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module DCM(CLKFB, CLKIN, DSSEN, PSCLK, PSEN, PSINCDEC, RST,
  CLK0, CLK90, CLK180, CLK270, CLK2X, CLK2X180, CLKDV, CLKFX, CLKFX180, LOCKED, PSDONE, STATUS); // synthesis syn_black_box
input CLKFB, CLKIN, DSSEN;
input PSCLK, PSEN, PSINCDEC, RST;
output CLK0, CLK90, CLK180, CLK270, CLK2X, CLK2X180;
output CLKDV, CLKFX, CLKFX180, LOCKED, PSDONE;
output [7:0] STATUS;
endmodule
