//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
//
/*
-------------------------------------------------------------------------------
   Title      : AUDIO FILTER DEMO DESIGN FOR THE XUP-V2Pro
   Project    : XUP-V2Pro 
-------------------------------------------------------------------------------
   File       : COLOR_CHAR_MODE_SVGA_CTRL.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/20
   Last Update: 2005/06/15
   Copyright  : (c) Xilinx Inc, 2005
-------------------------------------------------------------------------------
   Uses       : SVGA_TIMING_GENERATION.v DRIVE_DAC_DATA.v CHARACTER_MODE.v CLUT.v
				COLOR_RAM.v
-------------------------------------------------------------------------------
   Used by    : 
-------------------------------------------------------------------------------
   Description: This is the top level module for the color character mode SVGA display.
   	This design supports screen resolutions up to 800 x 600 (75 rows of 100 characters).
	This module uses 4 block RAMS to store the ASCII characters, 2 block RAMS to define
 	the color of the characters and a block RAM	as a character generator ROM.
    NO external RAMS are required. 

	Conventions:
		All external port signals are UPPER CASE.
		Active HIGH PORT signals have a P suffex
		Active LOW PORT signals have a N suffex.
		All internal signals are LOWER CASE and are active HIGH.

-------------------------------------------------------------------------------
   Revisions  :
   Date        Version  Author  	  Description
   2001/06/20  1.0      Ballantyne    Initial Release
   2005/06/15  1.1      Ballantyne    Disabled the sync on green
-------------------------------------------------------------------------------
*/
module color_char_mode_svga_ctrl (
VGA_OUT_PIXEL_CLOCK,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH,
VGA_VSYNCH,
VGA_OUT_RED,
VGA_OUT_GREEN,
VGA_OUT_BLUE,

bram_char_addr,
BRAM_Din,
char_read_enable,
bram_wen,

pixel_clock,
reset
);

output 			VGA_OUT_PIXEL_CLOCK;	// pixel clock for the video DAC
output			VGA_COMP_SYNCH_N;		// composite sync for the video DAC
output 			VGA_OUT_BLANK_N;		// composite blanking for the video DAC
output 			VGA_HSYNCH;				// horizontal sync for the VGA output connector
output			VGA_VSYNCH;				// vertical sync for the VGA output connector
output [7:0] 	VGA_OUT_RED;			// RED DAC data
output [7:0] 	VGA_OUT_GREEN;			// GREEN DAC data
output [7:0] 	VGA_OUT_BLUE;			// BLUE DAC data

output [0:31]	bram_char_addr;		// address of character to be read from processor BRAM
output [0:3]    bram_wen;
input [0:31]	BRAM_Din;		        // ASCII data of character from processor BRAM
output			char_read_enable;		// write enable
		/* synthesis syn_noclockbuf = 1 */// write clock

//input [3:0]		color_write_data;		// character color data

input	 		pixel_clock				/* synthesis syn_noclockbuf = 1 */; // pixel clock input 
input			reset;					// reset

wire			VGA_OUT_PIXEL_CLOCK;
wire 			VGA_COMP_SYNCH_N;
wire 			VGA_OUT_BLANK_N;
wire 			VGA_HSYNCH;
wire 			VGA_VSYNCH;
wire [7:0]		VGA_OUT_RED; 
wire [7:0]		VGA_OUT_GREEN; 
wire [7:0]		VGA_OUT_BLUE; 

wire [7:0]		bm_red_data;
wire [7:0]		bm_green_data; 
wire [7:0]		bm_blue_data; 

wire 			bm_display_active;
wire 			char_display_active;
wire [7:0]		dac_red_data;
wire [7:0]		dac_green_data;
wire [7:0]		dac_green;
wire [7:0]		dac_blue_data;
wire [7:0]		dac_blue;


wire 			h_synch_delay;
wire 			v_synch_delay;
wire 			comp_synch;
wire 			blank;
wire [2:0]		char_line_count;
wire [13:0]		char_mode_address;
wire 			char_mode_data;
wire [13:0]		char_address;
wire [2:0]		char_pixel;
wire [19:0]		vga_ram_read_address;

wire 			pixel_clock;
wire 			reset;

wire [7:0] 	red_data;
wire [7:0] 	green_data;
wire [7:0] 	blue_data;
wire		pixel_data;
wire [3:0] 	aligned_color_data;
wire [3:0] 	color_ram_data;

wire		color_read_clock;
wire [3:0] 	color_read_data;
wire [12:0] color_read_addr;
wire		color_read_enable;

wire		char_write_clock;
wire [3:0]  color_write_data;
wire [12:0] char_write_addr;
wire		char_write_enable;
wire [7:0]  ascii_code;

assign bram_wen = 4'b0000;
// make the internal connections
assign bm_red_data 		= red_data;
assign bm_green_data 	= green_data;
assign bm_blue_data 	= blue_data;

assign char_address [13:0] = char_mode_address[13:0];	// one spare bit for 1024 x 768 expansion

assign high  = 1'b1;
assign low  = 1'b0;
assign ascii_code = BRAM_Din[23:31];
assign color_ram_data = BRAM_Din[12:15];
assign bram_char_addr[17:29] = char_mode_address[12:0];
assign bram_char_addr[30:31] = 2'b00;
assign bram_char_addr[0:16] = 17'b00000000000000000;
assign color_read_clock = pixel_clock;
assign color_read_enable = 1'b1;
assign char_read_enable  = 1'b1;

assign color_read_addr[12:0] = char_mode_address[12:0];
assign pixel_data = char_mode_data;

OFDDRTRSE RED_0 (
.O   (VGA_OUT_RED[0]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[0]),
.D1  (dac_red_data[0]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE RED_1 (
.O   (VGA_OUT_RED[1]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[1]),
.D1  (dac_red_data[1]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE RED_2 (
.O   (VGA_OUT_RED[2]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[2]),
.D1  (dac_red_data[2]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE RED_3 (
.O   (VGA_OUT_RED[3]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[3]),
.D1  (dac_red_data[3]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE RED_4 (
.O   (VGA_OUT_RED[4]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[4]),
.D1  (dac_red_data[4]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE RED_5 (
.O   (VGA_OUT_RED[5]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[5]),
.D1  (dac_red_data[5]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE RED_6 (
.O   (VGA_OUT_RED[6]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[6]),
.D1  (dac_red_data[6]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE RED_7 (
.O   (VGA_OUT_RED[7]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_red_data[7]),
.D1  (dac_red_data[7]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE GREEN_0 (
.O   (VGA_OUT_GREEN[0]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[0]),
.D1  (dac_green_data[0]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE GREEN_1 (
.O   (VGA_OUT_GREEN[1]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[1]),
.D1  (dac_green_data[1]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE GREEN_2 (
.O   (VGA_OUT_GREEN[2]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[2]),
.D1  (dac_green_data[2]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE GREEN_3 (
.O   (VGA_OUT_GREEN[3]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[3]),
.D1  (dac_green_data[3]),
.R   (reset),
.S   (low),
.T   (low)
);
OFDDRTRSE GREEN_4 (
.O   (VGA_OUT_GREEN[4]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[4]),
.D1  (dac_green_data[4]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE GREEN_5 (
.O   (VGA_OUT_GREEN[5]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[5]),
.D1  (dac_green_data[5]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE GREEN_6 (
.O   (VGA_OUT_GREEN[6]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[6]),
.D1  (dac_green_data[6]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE GREEN_7 (
.O   (VGA_OUT_GREEN[7]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_green_data[7]),
.D1  (dac_green_data[7]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_0 (
.O   (VGA_OUT_BLUE[0]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[0]),
.D1  (dac_blue_data[0]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_1 (
.O   (VGA_OUT_BLUE[1]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[1]),
.D1  (dac_blue_data[1]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_2 (
.O   (VGA_OUT_BLUE[2]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[2]),
.D1  (dac_blue_data[2]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_3 (
.O   (VGA_OUT_BLUE[3]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[3]),
.D1  (dac_blue_data[3]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_4 (
.O   (VGA_OUT_BLUE[4]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[4]),
.D1  (dac_blue_data[4]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_5 (
.O   (VGA_OUT_BLUE[5]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[5]),
.D1  (dac_blue_data[5]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_6 (
.O   (VGA_OUT_BLUE[6]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[6]),
.D1  (dac_blue_data[6]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE BLUE_7 (
.O   (VGA_OUT_BLUE[7]),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (dac_blue_data[7]),
.D1  (dac_blue_data[7]),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE HSYNC (
.O   (VGA_HSYNCH),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (h_synch_delay),
.D1  (h_synch_delay),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE VSYNC (
.O   (VGA_VSYNCH),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (v_synch_delay),
.D1  (v_synch_delay),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE COMP_SYNC(
.O   (VGA_COMP_SYNCH_N),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (1'b0),	// disable sync on green
.D1  (1'b0),	// disable sync on green
.R   (low),
.S   (low),
.T   (low)
);

OFDDRTRSE BLANK(
.O   (VGA_OUT_BLANK_N),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (!blank),
.D1  (!blank),
.R   (reset),
.S   (low),
.T   (low)
);

OFDDRTRSE PIXEL_CLOCK(
.O   (VGA_OUT_PIXEL_CLOCK),
.C0  (pixel_clock),
.C1  (!pixel_clock),
.CE  (high),
.D0  (low),
.D1  (high),
.R   (reset),
.S   (low),
.T   (low)
);

/*
// make the external connections
// shut down the video output during  reset
always @ (posedge pixel_clock or posedge reset) begin
	if (reset) begin
		VGA_HSYNCH	 		<= 1'b1;
		VGA_VSYNCH 	 		<= 1'b1;
		VGA_COMP_SYNCH_N 	<= 1'b1;
		VGA_OUT_BLANK_N 	<= 1'b0;
		VGA_OUT_RED			<= 8'h00;
		VGA_OUT_GREEN		<= 8'h00;
		VGA_OUT_BLUE		<= 8'h00;
		end
	else begin
		VGA_HSYNCH	 		<= h_synch_delay;
		VGA_VSYNCH 	 		<= v_synch_delay;
		VGA_COMP_SYNCH_N 	<= !comp_synch;
		VGA_OUT_BLANK_N 	<= !blank;
		VGA_OUT_RED			<= dac_red_data;
		VGA_OUT_GREEN		<= dac_green_data;
		VGA_OUT_BLUE		<= dac_blue_data;
		end
	end

assign  VGA_OUT_PIXEL_CLOCK	= pixel_clock;
*/

// instantiate the SVGA timing generator
SVGA_TIMING_GENERATION SVGA_TIMING_GENERATION(
pixel_clock,
reset,
h_synch_delay,
v_synch_delay,
comp_synch,
blank,
char_line_count,
char_mode_address,
char_pixel
);

// instantiate the character RAM and character generator ROM
CHARACTER_MODE CHARACTER_MODE(
ascii_code,
char_address,
char_line_count,
char_pixel,
pixel_clock,
char_mode_data,
reset
);

// Replaced with ppc bram
/*
// instantiate the color RAM
COLOR_RAM COLOR_RAM (
color_read_clock,
color_read_data,
color_read_addr,
color_read_enable,
color_write_data,
char_write_addr,
char_write_clock,
char_write_enable);
*/

// instantiate the color pipeline
COLOR_PIPE COLOR_PIPE (
color_ram_data, 
pixel_clock, 
reset, 
aligned_color_data);

// instantiate the color look up table
CLUT CLUT (
aligned_color_data, 
pixel_data, 
pixel_clock, 
reset, 
dac_red_data, 
dac_green, 
dac_blue);
assign dac_blue_data = 8'hff;
assign dac_green_data = 8'hff;

endmodule //COLOR_CHAR_MODE_SVGA_CTRL

module OFDDRTRSE (O, C0, C1, CE, D0, D1, R, S, T); // synthesis syn_black_box
    output O;
    input  C0, C1, CE, D0, D1, R, S, T;
endmodule

