module color_mux (

idata,
ivalid,
isSprite,
icolor,
orgb_color,
ippu_clk,
reset

);

input[0:31] idata;
input ivalid;
input[3:0] icolor;
input ippu_clk;
input reset;
output[0:23] orgb_color;
reg[0:23] orgb_color;
reg [1:0] base_blue;
reg [1:0] base_red;
reg [1:0] base_green;
reg [5:0] palette;
reg [4:0] addr;
//assign palette = icolor;
reg [5:0] 	flopRam [0:31];
input isSprite ;


//select which color to use from all the palette data stored in the register
always @ (palette or isSprite or icolor) begin
  case ({isSprite, icolor})
    5'd0: palette = flopRam[0];
	 5'd1: palette = flopRam[1];
	 5'd2: palette = flopRam[2];
	 5'd3: palette = flopRam[3];
	 5'd5: palette = flopRam[5];
	 5'd6: palette = flopRam[6];
	 5'd7: palette = flopRam[7];
	 5'd9: palette = flopRam[9];
	 5'd10: palette = flopRam[10];
	 5'd11: palette = flopRam[11];
	 5'd13: palette = flopRam[13];
	 5'd14: palette = flopRam[14];
	 5'd15: palette = flopRam[15];
	 
	 5'd16: palette = flopRam[0];
	 5'd17: palette = flopRam[17];
	 5'd18: palette = flopRam[18];
	 5'd19: palette = flopRam[19];
	 5'd21: palette = flopRam[21];
	 5'd22: palette = flopRam[22];
	 5'd23: palette = flopRam[23];
	 5'd25: palette = flopRam[25];
	 5'd26: palette = flopRam[26];
	 5'd27: palette = flopRam[27];
	 5'd29: palette = flopRam[29];
	 5'd30: palette = flopRam[30];
	 5'd31: palette = flopRam[31];
	 default: palette = flopRam[0];
  endcase
end

always @ (posedge ippu_clk) begin
	if(reset) begin
	  addr <= 5'd0;
	end
	else begin
	  if (ivalid) begin
		 //store the palette data read from memory
	    addr <= addr + 4'd4;
		 flopRam[addr] <= idata[2:7];
		 flopRam[addr+1] <= idata[10:15];
		 flopRam[addr+2] <= idata[18:23];
		 flopRam[addr+3] <= idata[26:31];
	  end
	end
end

//decode the color base of the palette
always @ (palette[3:0]) begin
	case (palette[3:0])
    4'd0: base_blue = 2'b11;
	 
	 4'd1: base_blue = 2'b11;
	 4'd2: base_blue = 2'b00;
	 4'd3: base_blue = 2'b11;
	 4'd4: base_blue = 2'b00;
	 4'd5: base_blue = 2'b11;
	 4'd6: base_blue = 2'b00;
	 4'd7: base_blue = 2'b11;
	 4'd8: base_blue = 2'b00;
	 4'd9: base_blue = 2'b11;
	 4'd10: base_blue = 2'b00;
	 4'd11: base_blue = 2'b11;
	 4'd12: base_blue = 2'b00;
	 
	 
	 4'd13: base_blue = 2'b10;
	 4'd14: base_blue = 2'b01;
	 4'd15: base_blue = 2'b00;	
	 default: base_blue = 2'b00;
  endcase
end

//decode the color base of the palette
always @ (palette[3:0]) begin
	case (palette[3:0])
    4'd0: base_green = 2'b11;
	 
	 4'd1: base_green = 2'b11;
	 4'd2: base_green = 2'b11;
	 4'd3: base_green = 2'b11;
	 4'd4: base_green = 2'b11;
	 4'd5: base_green = 2'b00;
	 4'd6: base_green = 2'b00;
	 4'd7: base_green = 2'b00;
	 4'd8: base_green = 2'b00;
	 4'd9: base_green = 2'b11;
	 4'd10: base_green = 2'b11;
	 4'd11: base_green = 2'b11;
	 4'd12: base_green = 2'b11;
	 
	 4'd13: base_green = 2'b00;
	 4'd14: base_green = 2'b01;
	 4'd15: base_green = 2'b00;	
	 default: base_green = 2'b00;
  endcase
end

//decode the color base of the palette
always @ (palette[3:0]) begin
	case (palette[3:0])
    4'd0: base_red = 2'b11;
	 
	 4'd1: base_red = 2'b11;
	 4'd2: base_red = 2'b11;
	 4'd3: base_red = 2'b00;
	 4'd4: base_red = 2'b00; 
	 4'd5: base_red = 2'b11;
	 4'd6: base_red = 2'b11;
	 4'd7: base_red = 2'b00;
	 4'd8: base_red = 2'b00;
	 4'd9: base_red = 2'b11;
	 4'd10: base_red = 2'b11;
	 4'd11: base_red = 2'b00;
	 4'd12: base_red = 2'b00;
	 
	 4'd13: base_red = 2'b10;
	 4'd14: base_red = 2'b01;
	 4'd15: base_red = 2'b00;	
	 default: base_red = 2'b00;
  endcase
end	


//combine the brightness info in the palette bits(bits 5-4) for the final RGB output to the VGA controller
always @ (palette[5:4] or base_red or base_green or base_blue) begin
  case (palette[5:4])
    //darkest
    2'd0: orgb_color = {6'b0, base_red, 6'b0, base_green, 6'b0, base_blue};
	 2'd1: orgb_color = {4'b0,{2{base_red}},4'b0,{2{base_green}},4'b0,{2{base_blue}} };
	 2'd2: orgb_color = {2'b0,{3{base_red}},2'b0,{3{base_green}},2'b0,{3{base_blue}} };
	 //brightest
	 2'd3: orgb_color = {{4{base_red}}, {4{base_green}}, {4{base_blue}}};
	 default: orgb_color = 24'b0;
  endcase
end
	 

endmodule 