module ppu (

FSL_S_CLK,
FSL_S_READ,
FSL_S_DATA,
FSL_S_CONTROL,
FSL_S_EXISTS,

//sram_addr,
//fram_Dout,
sram_Din,
sram_en,
sram_wen,
sram_addr,
//fram_rst,
//fram_clk,

vram_addr,
vram_Din,
vram_en,
vram_wen,

rgb_Dout,
ppu_active,
ppu_pal_fetch,
ppu_sprite_attr,
ppu_sprite_pat,
ppu_sprite_active,
ppu_clock,
reset
);

output FSL_S_CLK;
output FSL_S_READ;
input [0:31] FSL_S_DATA;
input FSL_S_CONTROL;
input FSL_S_EXISTS;

output [0:23] rgb_Dout;
output [0:31] sram_addr;
input  [0:31] sram_Din;
output sram_en;
output [0:3] sram_wen;
//output fram_clk;
//output fram_rst;

output [0:31] vram_addr;
input [0:31] vram_Din;
output vram_en;
output [0:3] vram_wen;
input ppu_active;
input ppu_pal_fetch;
input ppu_sprite_attr;
input ppu_sprite_pat;
input ppu_sprite_active;
input ppu_clock;
input reset;

reg [8:0] pixel_count;
reg [8:0] line_count;
wire [5:0] tile_col;
wire [5:0] tile_row;
wire [13:0] nt_addr;
wire [13:0] at_addr;
wire [13:0] out_addr;
reg [13:0] out_addr1q;
//reg [13:0] out_addr2q;
//wire [0:31] fram_Dout;
//wire [0:31] fram_Dout;
reg [7:0] pt_byte0;
reg [15:0] pt_byte0_1q;
reg [15:0] pt_byte1;
reg [7:0] din_muxed;
reg [11:0] pt_addr;
reg [2:0] fetch_count;
reg [2:0] pix_remain;
reg [1:0] din_twobits;
reg [1:0] at_bits;
reg [1:0] at_bits1q;
reg [1:0] at_bits2q;
reg [15:0] at_bits3q;
reg pt0_muxed;
reg pt1_muxed;
reg [1:0]  at_bits_muxed;
reg [7:0] pt0_muxedq;
reg [7:0] pt1_muxedq;
wire [1:0] nt_num;
wire r_ppu_disable;
wire [0:23] rgb_color;
wire [3:0] pt_index;
wire [3:0] h_carry;
wire [9:0] r_vscroll_offset;
wire [9:0] r_hscroll_offset;
wire [1:0] wrap;
wire [6:0] row_pointer;
wire [13:5] pal_addr_base;
wire [7:0] sprite_attr_addr;
wire [13:0] sprite_pat_addr;
wire [3:0] sprite_pixel;
wire [3:0] pixel_color;
wire [1:0] mux_addr;
wire sprite_enable;
reg [2:0] pal_addr;
reg [13:0] nt_addrq;
reg pal_valid;
reg ppu_enable;
reg [13:0] sprite_pat_addrq;
assign mux_addr = ppu_sprite_pat ? sprite_pat_addrq[1:0] : out_addr1q[1:0];
assign pal_addr_base = {6'h3f, 3'b0};

//use different name table in scrolling
assign nt_num = {tile_row[5]& r_vscroll_offset[9], tile_col[5]& r_hscroll_offset[9]};
assign pt_index = line_count[3:1] + r_vscroll_offset[2:0];
assign h_carry = pixel_count[2:0] + r_hscroll_offset[2:0];

assign rgb_Dout = r_ppu_disable ? 24'b0 : rgb_color;

//ppu is divided into 32x30 tiles
assign tile_col = pixel_count[8:3] + r_hscroll_offset[8:3] + h_carry[3];
//special case for the row number to wrap around if it is greater then 29, for vertical scrolling
assign row_pointer = line_count[8:4] + r_vscroll_offset[8:3] + pt_index[3];
assign wrap = row_pointer > 7'd59 ? 3'b100 : 
					row_pointer > 7'd29 ? 3'b010 : 3'b000;
assign tile_row = line_count[8:4] + r_vscroll_offset[8:3] + wrap + pt_index[3];

//assign the memory address for reading
//could either be palette, pattern, name, sprite or attribute
assign vram_addr[18:29] = ppu_pal_fetch ? {pal_addr_base, pal_addr}:
									ppu_sprite_pat ? out_addr[13:2] : out_addr1q[13:2];
assign vram_addr[0:17] = 18'b0;
assign vram_addr[30:31] = 2'b0;
assign out_addr = ppu_sprite_pat ? sprite_pat_addr : 
						  fetch_count == 3'b101 ? nt_addrq : 
						  fetch_count == 3'b100 ? at_addr : {2'b0, pt_addr};
//name table address is based on the current tile row and tile column
assign nt_addr = {2'b10, nt_num, tile_row[4:0], tile_col[4:0]};
assign at_addr = {nt_addrq[13:10], 4'b1111, tile_row[4:2], tile_col[4:2]};


assign sram_en = ppu_sprite_attr;
assign sram_wen = 4'b0000;
assign vram_en = ppu_active | ppu_sprite_pat | ppu_pal_fetch;
assign vram_wen = 4'b0000;
assign sram_addr = {24'b0, sprite_attr_addr[7:0]};

//choose between sprite or background
assign pixel_color = (sprite_pixel[1:0] == 2'b00 | !sprite_enable) ? {at_bits_muxed, pt1_muxed, pt0_muxed} : sprite_pixel;



//select the starting bit of the pattern to output base on the 
//horizontial scrolling offset
always @ (r_hscroll_offset[2:0] or pt_byte0_1q) begin
  case (r_hscroll_offset[2:0])
    3'b000: pt0_muxed = pt_byte0_1q[15];
	 3'b001: pt0_muxed = pt_byte0_1q[14];
	 3'b010: pt0_muxed = pt_byte0_1q[13];
	 3'b011: pt0_muxed = pt_byte0_1q[12];
	 3'b100: pt0_muxed = pt_byte0_1q[11];
	 3'b101: pt0_muxed = pt_byte0_1q[10];
	 3'b110: pt0_muxed = pt_byte0_1q[9];
	 3'b111: pt0_muxed = pt_byte0_1q[8];
	 default: pt0_muxed = 1'b0;
  endcase
end

//select the starting bit of the pattern to output base on the 
//horizontial scrolling offset
always @ (r_hscroll_offset[2:0] or at_bits3q) begin
  case (r_hscroll_offset[2:0])
    3'b000: at_bits_muxed = at_bits3q[15:14];
	 3'b001: at_bits_muxed = at_bits3q[13:12];
	 3'b010: at_bits_muxed = at_bits3q[11:10];
	 3'b011: at_bits_muxed = at_bits3q[9:8];
	 3'b100: at_bits_muxed = at_bits3q[7:6];
	 3'b101: at_bits_muxed = at_bits3q[5:4];
	 3'b110: at_bits_muxed = at_bits3q[3:2];
	 3'b111: at_bits_muxed = at_bits3q[1:0];
	 default: at_bits_muxed = 2'b0;
  endcase
end

//select the starting bit of the pattern to output base on the 
//horizontial scrolling offset
always @ (r_hscroll_offset[2:0] or pt_byte0_1q) begin
  case (r_hscroll_offset[2:0])
    3'b000: pt1_muxed = pt_byte1[15];
	 3'b001: pt1_muxed = pt_byte1[14];
	 3'b010: pt1_muxed = pt_byte1[13];
	 3'b011: pt1_muxed = pt_byte1[12];
	 3'b100: pt1_muxed = pt_byte1[11];
	 3'b101: pt1_muxed = pt_byte1[10];
	 3'b110: pt1_muxed = pt_byte1[9];
	 3'b111: pt1_muxed = pt_byte1[8];
	 default: pt1_muxed = 1'b0;
  endcase
end

//muxed the incoming 32bits data to 8bits data base on the memory address
always @ (out_addr1q or vram_Din) begin
  case (mux_addr)
    2'b11: din_muxed = vram_Din[24:31];
	 2'b10: din_muxed = vram_Din[16:23];
	 2'b01: din_muxed = vram_Din[8:15];
	 2'b00: din_muxed = vram_Din[0:7];
	 default: din_muxed = 8'b0;
  endcase
end

//select the two attribute bits depending on which 
//16x16 area the PPU is currently in
always @ (tile_col[1] or tile_row[1] or din_muxed) begin
  case ({tile_row[1], tile_col[1]})
    2'b00: din_twobits = din_muxed[1:0];
	 2'b01: din_twobits = din_muxed[3:2];
	 2'b10: din_twobits = din_muxed[5:4];
	 2'b11: din_twobits = din_muxed[7:6];
	 default: din_twobits = 2'b0;
  endcase
end 

always @ (posedge ppu_clock) begin
  if (reset) begin
    pix_remain <= 3'b0;
    pixel_count <= 9'b0;
	 line_count <= 9'b0;
	// tile_col <= 5'b0;
	// tile_row <= 5'b0;
	 pt_addr <= 12'b0;
	 nt_addrq <= 14'b0;
	 pt_byte0 <= 8'b0;
	 pt_byte0_1q <= 8'b0;
	 pt_byte1 <= 8'b0;
	 fetch_count <= 3'b0;
	 out_addr1q <= 13'b0;
	 pt0_muxedq <= 8'b0;
	 pt1_muxedq <= 8'b0;
	 ppu_enable <= 1'b0;
	 at_bits <= 2'b0;
	 at_bits1q <= 2'b0;
    at_bits2q <= 2'b0;
	 pal_addr <= 3'b0;
	 pal_valid <= 1'b0;
	 at_bits3q <= 16'b0;
	 sprite_pat_addrq <= 14'b0;
  end
  else begin
  
  if (ppu_pal_fetch)
    pal_addr <= pal_addr + 1'b1;
  
  //if (ppu_active)
    ppu_enable <= !ppu_enable;
  //else 
   // ppu_enable <= 1'b0;

  if (ppu_active) begin
	 if (ppu_enable) 
	   pixel_count <= pixel_count + 1'b1;
  end
  else 
	 pixel_count <= 0;
	 
  //enable every two cycle, each pixel and line is output twice
  //to double the display size from 256x240 to 512x480
  if (ppu_enable) begin
		
    if (pixel_count == 9'd263) begin //fetch an extra 8 pixels for horizontal scrolling
	   if (line_count[8:1] == 8'd239)
		  line_count <= 9'b0;
      else
		  line_count <= line_count + 1'b1;		  
	 end
	 
	 //fetch the next 8 pixels after 3 pixels
    //fetch takes 5 cycle to complete	 
	 if (pixel_count[2:0] == 3'b010) begin
		//start fetching from block ram
		//fetch_count = 5, request for name table data
		//fetch_count = 4, request for attribute table data, name table data valid
		//fetch_count = 3, request for bit0 of background pattern, attribute data valid
		//fetch_count = 2, request for bit1 of background pattern, bit0 pattern data valid
		//fecth_count = 1, bit1 pattern data valid
		fetch_count <= 3'b101;
	 end
	 
	 if (pixel_count[2:0] == 3'b000) 
	   nt_addrq <= nt_addr;
	 
	 if (fetch_count != 3'b0) 
	   fetch_count <= fetch_count - 1'b1;
	 
	 if (fetch_count == 3'b100) 
	   pt_addr <= (din_muxed << 4) + pt_index[2:0];
	 else begin
	   if (fetch_count == 3'b011) begin
	     pt_addr <= pt_addr + 4'd8;
	   end
	end
	 
	 if (fetch_count == 3'b011)
	   at_bits <= din_twobits;
		
	
	 if (fetch_count == 3'b010) 
	   pt_byte0 <= din_muxed;
		
    if (fetch_count == 3'b001)
	   pt_byte0_1q[7:0] <= pt_byte0;
	 else 
	   pt_byte0_1q[7:0] <= {pt_byte0_1q[6:0],1'b0};
		
	 if (fetch_count == 3'b001) 
	   pt_byte1[7:0] <= din_muxed;
    else 
	   pt_byte1[7:0] <= {pt_byte1[6:0],1'b0};
		
	 //delay the at_bits for 2 cycle
    at_bits1q <= at_bits;
    at_bits2q <= at_bits1q;
	 
	 //shift registers
	 at_bits3q <= {at_bits3q[13:0], at_bits2q};
	 pt_byte0_1q[15:8] <= pt_byte0_1q[14:7];
	 pt_byte1[15:8] <= pt_byte1[14:7];
	 
	 //delay address for 1 cycle, for muxing the data coming in
	 out_addr1q <= out_addr;
	 
	 end
	 
	 pal_valid <= ppu_pal_fetch;
	 sprite_pat_addrq <= sprite_pat_addr;
  end
end


color_mux color_mux_inst0 (

.idata(vram_Din),
.isSprite(sprite_pixel[1:0] != 2'b00 & sprite_enable),
.ivalid(pal_valid),
.icolor(pixel_color),
.orgb_color(rgb_color),
.ippu_clk(ppu_clock),
.reset(reset)

);

ppu_ctrl ppu_ctrl_inst0 (

.FSL_S_CLK(FSL_S_CLK),
.FSL_S_READ(FSL_S_READ),
.FSL_S_DATA(FSL_S_DATA),
.FSL_S_CONTROL(FSL_S_CONTROL),
.FSL_S_EXISTS(FSL_S_EXISTS),
.ippu_clk(ppu_clock),
.ippu_reset(reset),
.oh_scroll(r_hscroll_offset),
.ov_scroll(r_vscroll_offset),
.oppu_disable(r_ppu_disable)

);


sprite_ctrl sprite_ctrl_inst0 (

.iat_data(sram_Din),
.isprite_active(ppu_sprite_active),
.ipt_data(din_muxed),
.osram_addr(sprite_attr_addr),
.ovram_addr(sprite_pat_addr),
.osprite_pixel(sprite_pixel),
.osprite_enable(sprite_enable),
.iat_fetch(ppu_sprite_attr),
.ipt_fetch(ppu_sprite_pat),
.ippu_clk(ppu_clock),
.reset(reset)

);
endmodule
